@@ -87,17 +87,10 @@ static int ratbagd_button_set_button(sd_bus *bus,
 	r = ratbag_button_set_button(button->lib_button, map);
 
 	if (r == 0) {
-		sd_bus *bus = sd_bus_message_get_bus(m);
 		sd_bus_emit_properties_changed(bus,
 					       button->path,
 					       RATBAGD_NAME_ROOT ".Button",
-					       "ButtonMapping",
-					       NULL);
-
-		sd_bus_emit_properties_changed(bus,
-					       button->path,
-					       RATBAGD_NAME_ROOT ".Button",
-					       "ActionType",
+					       "Mapping",
 					       NULL);
 	}
 
@@ -146,17 +139,10 @@ static int ratbagd_button_set_special(sd_bus *bus,
 	r = ratbag_button_set_special(button->lib_button, special);
 
 	if (r == 0) {
-		sd_bus *bus = sd_bus_message_get_bus(m);
 		sd_bus_emit_properties_changed(bus,
 					       button->path,
 					       RATBAGD_NAME_ROOT ".Button",
-					       "SpecialMapping",
-					       NULL);
-
-		sd_bus_emit_properties_changed(bus,
-					       button->path,
-					       RATBAGD_NAME_ROOT ".Button",
-					       "ActionType",
+					       "Mapping",
 					       NULL);
 	}
 
@@ -261,17 +247,10 @@ static int ratbagd_button_set_macro(sd_bus *bus,
 	}
 
 	if (r == 0) {
-		sd_bus *bus = sd_bus_message_get_bus(m);
 		sd_bus_emit_properties_changed(bus,
 					       button->path,
 					       RATBAGD_NAME_ROOT ".Button",
-					       "Macro",
-					       NULL);
-
-		sd_bus_emit_properties_changed(bus,
-					       button->path,
-					       RATBAGD_NAME_ROOT ".Button",
-					       "ActionType",
+					       "Mapping",
 					       NULL);
 	}
 
