@@ -32,4 +32,7 @@ void ratbagd_init_test_device(struct ratbagd *ctx);
 int ratbagd_reset_test_device(sd_bus_message *m,
 			      void *userdata,
 			      sd_bus_error *error);
+int ratbagd_load_test_device(sd_bus_message *m,
+			     void *userdata,
+			     sd_bus_error *error);
 #endif /* RATBAG_DEVELOPER_EDITION */
