@@ -957,7 +957,7 @@ hidpp20drv_read_resolution_dpi(struct ratbag_profile *profile)
 
 	if (drv_data->capabilities & HIDPP_CAP_ADJUSTIBLE_REPORT_RATE_8060) {
 		rc = hidpp20drv_read_report_rate_8060(device);
-		if (rc < 0)
+		if (rc < 0 && drv_data->report_rates[0] == 0)
 			return rc;
 
 		ratbag_profile_set_report_rate_list(profile,
@@ -1096,6 +1096,7 @@ hidpp20drv_read_special_key_mouse(struct ratbag_device *device)
 	drv_data->controls = NULL;
 	drv_data->num_controls = 0;
 	rc = hidpp20_special_key_mouse_get_controls(drv_data->dev, &drv_data->controls);
+	log_raw(device->ratbag, "num_control %d\n", rc);
 	if (rc > 0) {
 		drv_data->num_controls = rc;
 		rc = 0;
@@ -1208,9 +1209,7 @@ hidpp20drv_read_profile_8100(struct ratbag_profile *profile)
 
 	profile->is_enabled = drv_data->profiles->profiles[profile->index].enabled;
 
-	profile->is_active = false;
-	if ((int)profile->index == hidpp20drv_current_profile(device))
-		profile->is_active = true;
+	profile->is_active = (profile->index == drv_data->profiles->active_profile_index);
 
 	if (profile->is_active)
 		dpi_index = hidpp20_onboard_profiles_get_current_dpi_index(drv_data->dev);
@@ -1604,7 +1603,7 @@ hidpp20_log(void *userdata, enum hidpp_log_priority priority, const char *format
 {
 	struct ratbag_device *device = userdata;
 
-	log_msg_va(device->ratbag, priority, format, args);
+	log_msg_va(device->ratbag, (enum ratbag_log_priority)priority, format, args);
 }
 
 static void
@@ -1642,6 +1641,14 @@ hidpp20drv_init_device(struct ratbag_device *device,
 	if (num >= 0)
 		drv_data->num_leds = num;
 
+	num = ratbag_device_data_hidpp20_get_button_count(device->data);
+	if (num >= 0)
+		drv_data->num_buttons = num;
+
+	num = ratbag_device_data_hidpp20_get_report_rate(device->data);
+	if (num >= 0)
+		drv_data->report_rates[0] = num;
+
 	ratbag_device_init_profiles(device,
 				    drv_data->num_profiles,
 				    drv_data->num_resolutions,
