@@ -58,6 +58,7 @@ hidpp20_feature_get_name(uint16_t feature)
 	CASE_RETURN_STRING(HIDPP_PAGE_RESET);
 	CASE_RETURN_STRING(HIDPP_PAGE_BATTERY_LEVEL_STATUS);
 	CASE_RETURN_STRING(HIDPP_PAGE_BATTERY_VOLTAGE);
+	CASE_RETURN_STRING(HIDPP_PAGE_LED_SW_CONTROL);
 	CASE_RETURN_STRING(HIDPP_PAGE_KBD_REPROGRAMMABLE_KEYS);
 	CASE_RETURN_STRING(HIDPP_PAGE_SPECIAL_KEYS_BUTTONS);
 	CASE_RETURN_STRING(HIDPP_PAGE_WIRELESS_DEVICE_STATUS);
@@ -2082,7 +2083,7 @@ hidpp20_onboard_profiles_set_onboard_mode(struct hidpp20_device *device,
 		.msg.report_id = REPORT_ID_SHORT,
 		.msg.device_idx = device->index,
 		.msg.address = CMD_ONBOARD_PROFILES_SET_ONBOARD_MODE,
-		.msg.parameters[1] = onboard_mode,
+		.msg.parameters[0] = onboard_mode,
 	};
 
 	feature_index = hidpp_root_get_feature_idx(device,
@@ -2102,7 +2103,7 @@ hidpp20_onboard_profiles_set_onboard_mode(struct hidpp20_device *device,
 int
 hidpp20_onboard_profiles_get_current_profile(struct hidpp20_device *device)
 {
-	uint8_t feature_index;
+	uint8_t feature_index, unknown_0, active_profile_index;
 	int rc;
 	union hidpp20_message msg = {
 		.msg.report_id = REPORT_ID_SHORT,
@@ -2121,7 +2122,15 @@ hidpp20_onboard_profiles_get_current_profile(struct hidpp20_device *device)
 	if (rc)
 		return rc;
 
-	return msg.msg.parameters[1];
+	unknown_0 = msg.msg.parameters[0];
+	active_profile_index = msg.msg.parameters[1];
+
+	hidpp_log_raw(
+	    &device->base,
+	    "current active profile is index: %d unknown_0: %d\n",
+	    active_profile_index, unknown_0);
+
+	return active_profile_index;
 }
 
 int
@@ -2243,8 +2252,7 @@ hidpp20_onboard_profiles_allocate(struct hidpp20_device *device,
 {
 	struct hidpp20_onboard_profiles_info info = { 0 };
 	struct hidpp20_profiles *profiles;
-	int onboard_mode;
-	int rc;
+	int onboard_mode, active_profile_index, rc;
 
 	rc = hidpp20_onboard_profiles_get_profiles_desc(device, &info);
 	if (rc)
@@ -2268,6 +2276,12 @@ hidpp20_onboard_profiles_allocate(struct hidpp20_device *device,
 			return rc;
 	}
 
+	rc = hidpp20_onboard_profiles_get_current_profile(device);
+	if (rc < 0)
+		return rc;
+
+	active_profile_index = rc;
+
 	profiles = zalloc(sizeof(struct hidpp20_profiles));
 	profiles->profiles = zalloc(info.profile_count * sizeof(struct hidpp20_profile));
 	profiles->sector_size = info.sector_size;
@@ -2279,6 +2293,7 @@ hidpp20_onboard_profiles_allocate(struct hidpp20_device *device,
 	profiles->num_leds = HIDPP20_LED_COUNT;
 	profiles->has_g_shift = (info.mechanical_layout & 0x03) == 0x02;
 	profiles->has_dpi_shift = ((info.mechanical_layout & 0x0c) >> 2) == 0x02;
+	profiles->active_profile_index = active_profile_index;
 	switch(info.various_info & 0x07) {
 	case 1:
 		profiles->corded = 1;
