@@ -30,10 +30,14 @@
 #include <glib.h>
 #include <limits.h>
 
+#include "asus.h"
+#include "driver-sinowealth.h"
+#include "driver-steelseries.h"
 #include "libratbag.h"
 #include "libratbag-private.h"
 #include "libratbag-data.h"
 #include "hidpp20.h"
+#include "shared-macro.h"
 #include "usb-ids.h"
 
 #define GROUP_DEVICE "Device"
@@ -47,11 +51,14 @@ enum driver {
 	HIDPP10,
 	HIDPP20,
 	ROCCAT,
+	ROCCAT_KONE_PURE,
+	ROCCAT_EMP,
 	ETEKCITY,
 	GSKILL,
 	LOGITECH_G300,
 	LOGITECH_G600,
 	STEELSERIES,
+	ASUS,
 	SINOWEALTH,
 	SINOWEALTH_NUBWO,
 	OPENINPUT,
@@ -61,6 +68,8 @@ struct data_hidpp20 {
 	int index;
 	enum hidpp20_quirk quirk;
 	int led_count;
+	int report_rate;
+	int button_count;
 };
 
 struct data_hidpp10 {
@@ -73,6 +82,10 @@ struct data_hidpp10 {
 	int led_count;
 };
 
+struct data_sinowealth {
+	struct list supported_devices;
+};
+
 struct data_steelseries {
 	int device_version;
 	int button_count;
@@ -80,8 +93,18 @@ struct data_steelseries {
 	struct dpi_list *dpi_list;
 	struct dpi_range *dpi_range;
 	int macro_length;
-	int mono_led;
-	int short_button;
+	enum steelseries_quirk quirk;
+};
+
+struct data_asus {
+	int profile_count;
+	int button_count;
+	int8_t button_mapping[ASUS_MAX_NUM_BUTTON];
+	int led_count;
+	int dpi_count;
+	int is_wireless;
+	struct dpi_range *dpi_range;
+	uint32_t quirks;
 };
 
 struct ratbag_device_data {
@@ -94,7 +117,9 @@ struct ratbag_device_data {
 	union {
 		struct data_hidpp20 hidpp20;
 		struct data_hidpp10 hidpp10;
+		struct data_sinowealth sinowealth;
 		struct data_steelseries steelseries;
+		struct data_asus asus;
 	};
 
 	enum ratbag_led_type led_types[20];
@@ -161,9 +186,18 @@ init_data_hidpp20(struct ratbag *ratbag,
 	int num;
 	char *str;
 
+	data->hidpp20.button_count = -1;
 	data->hidpp20.index = -1;
 	data->hidpp20.led_count = -1;
+	data->hidpp20.report_rate = -1;
 
+	num = g_key_file_get_integer(keyfile, group, "Buttons", &error);
+	if (num > 0 || !error)
+		data->hidpp20.button_count = num;
+	if (error)
+		g_error_free(error);
+
+	error = NULL;
 	num = g_key_file_get_integer(keyfile, group, "DeviceIndex", &error);
 	if (num != 0 || !error)
 		data->hidpp20.index = num;
@@ -177,6 +211,13 @@ init_data_hidpp20(struct ratbag *ratbag,
 	if (error)
 		g_error_free(error);
 
+	error = NULL;
+	num = g_key_file_get_integer(keyfile, group, "ReportRate", &error);
+	if (num > 0 || !error)
+		data->hidpp20.report_rate = num;
+	if (error)
+		g_error_free(error);
+
 	str = g_key_file_get_string(keyfile, group, "Quirk", NULL);
 	data->hidpp20.quirk = HIDPP20_QUIRK_NONE;
 	if (str) {
@@ -187,6 +228,56 @@ init_data_hidpp20(struct ratbag *ratbag,
 	}
 }
 
+static void
+init_data_sinowealth(struct ratbag *ratbag,
+		  GKeyFile *keyfile,
+		  struct ratbag_device_data *data)
+{
+	const char *devices_group = "Driver/sinowealth/devices/";
+
+	GError *error = NULL;
+
+	size_t group_count = 0;
+	_cleanup_(g_strfreevp) char **groups = g_key_file_get_groups(keyfile, &group_count);
+
+	list_init(&data->sinowealth.supported_devices);
+
+	for (size_t i = 0; i < group_count; ++i) {
+		const char *device_group = groups[i];
+		if (startswith(device_group, devices_group) == NULL)
+			continue;
+
+		struct sinowealth_device_data *device = zalloc(sizeof(struct sinowealth_device_data));
+
+		device->button_count = g_key_file_get_integer(keyfile, device_group, "ButtonCount", &error);
+		g_clear_error(&error);
+
+		device->device_name = g_key_file_get_string(keyfile, device_group, "DeviceName", &error);
+		g_clear_error(&error);
+
+		device->fw_version = g_key_file_get_string(keyfile, device_group, "FwVersion", &error);
+		g_clear_error(&error);
+
+		_cleanup_free_ char *led_type_str = g_key_file_get_string(keyfile, device_group, "LedType", &error);
+		if (led_type_str) {
+			if (streq(led_type_str, "RGB")) {
+				device->led_type = SINOWEALTH_LED_TYPE_RGB;
+			} else if (streq(led_type_str, "RBG")) {
+				device->led_type = SINOWEALTH_LED_TYPE_RBG;
+			} else if (streq(led_type_str, "None")) {
+				device->led_type = SINOWEALTH_LED_TYPE_NONE;
+			} else {
+				log_error(ratbag, "Unknown LED type '%s' in group '%s'\n", led_type_str, device_group);
+
+				device->led_type = SINOWEALTH_LED_TYPE_NONE;
+			}
+		}
+		g_clear_error(&error);
+
+		list_insert(&data->sinowealth.supported_devices, &device->link);
+	}
+}
+
 static void
 init_data_steelseries(struct ratbag *ratbag,
 		  GKeyFile *keyfile,
@@ -194,7 +285,8 @@ init_data_steelseries(struct ratbag *ratbag,
 {
 	const char *group = "Driver/steelseries";
 	GError *error = NULL;
-	_cleanup_(freep) char *str = NULL;
+	_cleanup_(freep) char *dpi_range = NULL;
+	_cleanup_(freep) char *quirk = NULL;
 	int num;
 
 	data->steelseries.device_version = -1;
@@ -202,8 +294,7 @@ init_data_steelseries(struct ratbag *ratbag,
 	data->steelseries.led_count = -1;
 	data->steelseries.dpi_list = NULL;
 	data->steelseries.dpi_range = NULL;
-	data->steelseries.mono_led = 0;
-	data->steelseries.short_button = 0;
+	data->steelseries.quirk = STEELSERIES_QURIK_NONE;
 
 	num = g_key_file_get_integer(keyfile, group, "Buttons", &error);
 	if (num != 0 || !error)
@@ -225,13 +316,13 @@ init_data_steelseries(struct ratbag *ratbag,
 	if (error)
 		g_error_free(error);
 
-	str = g_key_file_get_string(keyfile, group, "DpiRange", NULL);
-	if (str) {
-		data->steelseries.dpi_range = dpi_range_from_string(str);
+	dpi_range = g_key_file_get_string(keyfile, group, "DpiRange", NULL);
+	if (dpi_range) {
+		data->steelseries.dpi_range = dpi_range_from_string(dpi_range);
 	} else {
-		str = g_key_file_get_string(keyfile, group, "DpiList", NULL);
-		if (str)
-			data->steelseries.dpi_list = dpi_list_from_string(str);
+		dpi_range = g_key_file_get_string(keyfile, group, "DpiList", NULL);
+		if (dpi_range)
+			data->steelseries.dpi_list = dpi_list_from_string(dpi_range);
 	}
 
 	error = NULL;
@@ -241,19 +332,86 @@ init_data_steelseries(struct ratbag *ratbag,
 	if (error)
 		g_error_free(error);
 
-	error = NULL;
-	num = g_key_file_get_integer(keyfile, group, "MonoLed", &error);
-	if (num > 0 || !error)
-		data->steelseries.mono_led = num;
-	if (error)
-		g_error_free(error);
+	quirk = g_key_file_get_string(keyfile, group, "Quirk", NULL);
+	if (quirk) {
+		if (streq(quirk, "Rival100"))
+			data->steelseries.quirk = STEELSERIES_QUIRK_RIVAL100;
+		if (streq(quirk, "SenseiRAW"))
+			data->steelseries.quirk = STEELSERIES_QUIRK_SENSEIRAW;
+	}
+}
 
-	error = NULL;
-	num = g_key_file_get_integer(keyfile, group, "ShortButton", &error);
-	if (num > 0 || !error)
-		data->steelseries.short_button = num;
-	if (error)
-		g_error_free(error);
+static void
+init_data_asus(struct ratbag *ratbag,
+		GKeyFile *keyfile,
+		struct ratbag_device_data *data)
+{
+	const char *group = "Driver/asus";
+	GError *error = NULL;
+
+	data->asus.profile_count = -1;
+	data->asus.button_count = -1;
+	data->asus.led_count = -1;
+	data->asus.dpi_count = -1;
+	data->asus.dpi_range = NULL;
+	data->asus.is_wireless = -1;
+	data->asus.quirks = 0;
+	for (unsigned int i = 0; i < ASUS_MAX_NUM_BUTTON; i++)
+		data->asus.button_mapping[i] = -1;
+
+	int profiles = g_key_file_get_integer(keyfile, group, "Profiles", &error);
+	if (!error && profiles >= 0)
+		data->asus.profile_count = profiles;
+	g_clear_error(&error);
+
+	int buttons = g_key_file_get_integer(keyfile, group, "Buttons", &error);
+	if (!error && buttons >= 0 && buttons <= ASUS_MAX_NUM_BUTTON)
+		data->asus.button_count = buttons;
+	g_clear_error(&error);
+
+	gsize button_mapping_count = 0;
+	_cleanup_(g_strfreevp) char **button_mapping = g_key_file_get_string_list(keyfile, group, "ButtonMapping", &button_mapping_count, &error);
+	if (!error && button_mapping) {
+		for (unsigned int i = 0; i < button_mapping_count; i++) {
+			data->asus.button_mapping[i] = (int8_t) strtoul(button_mapping[i], NULL, 10);
+		}
+	}
+	g_clear_error(&error);
+
+        int leds = g_key_file_get_integer(keyfile, group, "Leds", &error);
+	if (!error && leds >= 0 && leds <= ASUS_MAX_NUM_LED)
+		data->asus.led_count = leds;
+	g_clear_error(&error);
+
+        int dpis = g_key_file_get_integer(keyfile, group, "Dpis", &error);
+	if (!error && dpis >= 2 && dpis <= ASUS_MAX_NUM_DPI)
+		data->asus.dpi_count = dpis;
+	g_clear_error(&error);
+
+	_cleanup_(freep) char *dpi_range = g_key_file_get_string(keyfile, group, "DpiRange", &error);
+	if (!error && dpi_range)
+		data->asus.dpi_range = dpi_range_from_string(dpi_range);
+	g_clear_error(&error);
+
+        int wireless = g_key_file_get_integer(keyfile, group, "Wireless", &error);
+	if (!error && (wireless == 0 || wireless == 1))
+		data->asus.is_wireless = wireless;
+	g_clear_error(&error);
+
+        gsize quirks_count = 0;
+	_cleanup_(g_strfreevp) char **quirks = g_key_file_get_string_list(keyfile, group, "Quirks", &quirks_count, &error);
+	if (!error && quirks) {
+		for (unsigned int i = 0; i < quirks_count; i++) {
+			if (streq(quirks[i], "DOUBLE_DPI")) {
+				data->asus.quirks |= ASUS_QUIRK_DOUBLE_DPI;
+			} else if (streq(quirks[i], "STRIX_PROFILE")) {
+				data->asus.quirks |= ASUS_QUIRK_STRIX_PROFILE;
+			} else {
+				log_debug(ratbag, "%s is invalid quirk. Ignoring...\n", quirks[i]);
+			}
+		}
+	}
+	g_clear_error(&error);
 }
 
 static const struct driver_map {
@@ -266,12 +424,15 @@ static const struct driver_map {
 	{ HIDPP10, "hidpp10", init_data_hidpp10 },
 	{ HIDPP20, "hidpp20", init_data_hidpp20 },
 	{ ROCCAT, "roccat", NULL },
+	{ ROCCAT_KONE_PURE, "roccat-kone-pure", NULL },
+	{ ROCCAT_EMP, "roccat-kone-emp", NULL },
 	{ ETEKCITY, "etekcity", NULL},
 	{ GSKILL, "gskill", NULL },
 	{ LOGITECH_G300, "logitech_g300", NULL},
 	{ LOGITECH_G600, "logitech_g600", NULL},
 	{ STEELSERIES, "steelseries", init_data_steelseries },
-	{ SINOWEALTH, "sinowealth", NULL },
+	{ ASUS, "asus", init_data_asus },
+	{ SINOWEALTH, "sinowealth", init_data_sinowealth },
 	{ SINOWEALTH_NUBWO, "sinowealth_nubwo", NULL},
 	{ OPENINPUT, "openinput", NULL },
 };
@@ -309,8 +470,34 @@ ratbag_device_data_destroy(struct ratbag_device_data *data)
 {
 	switch (data->drivertype) {
 	case HIDPP10:
+		if (data->hidpp10.dpi_list) {
+			free(data->hidpp10.dpi_list->entries);
+			free(data->hidpp10.dpi_list);
+		}
+
+		free(data->hidpp10.dpi_range);
 		free(data->hidpp10.profile_type);
 		break;
+	case SINOWEALTH: {
+		struct sinowealth_device_data *device_data = NULL;
+		struct sinowealth_device_data *device_data_next = NULL;
+
+		list_for_each_safe(device_data, device_data_next, &data->sinowealth.supported_devices, link) {
+			free(device_data->device_name);
+			free(device_data->fw_version);
+			free(device_data);
+		}
+
+		break;
+	}
+	case STEELSERIES:
+		if (data->steelseries.dpi_list) {
+			free(data->steelseries.dpi_list->entries);
+			free(data->steelseries.dpi_list);
+		}
+
+		free(data->steelseries.dpi_range);
+		break;
 	default:
 		break;
 	}
@@ -594,6 +781,14 @@ ratbag_device_data_hidpp20_get_index(const struct ratbag_device_data *data)
 	return data->hidpp20.index;
 }
 
+int
+ratbag_device_data_hidpp20_get_button_count(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == HIDPP20);
+
+	return data->hidpp20.button_count;
+}
+
 int
 ratbag_device_data_hidpp20_get_led_count(const struct ratbag_device_data *data)
 {
@@ -602,6 +797,14 @@ ratbag_device_data_hidpp20_get_led_count(const struct ratbag_device_data *data)
 	return data->hidpp20.led_count;
 }
 
+int
+ratbag_device_data_hidpp20_get_report_rate(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == HIDPP20);
+
+	return data->hidpp20.report_rate;
+}
+
 enum hidpp20_quirk
 ratbag_device_data_hidpp20_get_quirk(const struct ratbag_device_data *data)
 {
@@ -610,6 +813,16 @@ ratbag_device_data_hidpp20_get_quirk(const struct ratbag_device_data *data)
 	return data->hidpp20.quirk;
 }
 
+/* SinoWealth */
+
+const struct list *
+ratbag_device_data_sinowealth_get_supported_devices(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == SINOWEALTH);
+
+	return &data->sinowealth.supported_devices;
+}
+
 /* SteelSeries */
 
 int
@@ -660,18 +873,68 @@ ratbag_device_data_steelseries_get_macro_length(const struct ratbag_device_data
 	return data->steelseries.macro_length;
 }
 
-int
-ratbag_device_data_steelseries_get_mono_led(const struct ratbag_device_data *data)
+enum steelseries_quirk
+ratbag_device_data_steelseries_get_quirk(const struct ratbag_device_data *data)
 {
 	assert(data->drivertype == STEELSERIES);
 
-	return data->steelseries.mono_led;
+	return data->steelseries.quirk;
+}
+
+/* ASUS */
+
+int
+ratbag_device_data_asus_get_profile_count(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == ASUS);
+	return data->asus.profile_count;
 }
 
 int
-ratbag_device_data_steelseries_get_short_button(const struct ratbag_device_data *data)
+ratbag_device_data_asus_get_button_count(const struct ratbag_device_data *data)
 {
-	assert(data->drivertype == STEELSERIES);
+	assert(data->drivertype == ASUS);
+	return data->asus.button_count;
+}
 
-	return data->steelseries.short_button;
+const int8_t *
+ratbag_device_data_asus_get_button_mapping(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == ASUS);
+	return data->asus.button_mapping;
+}
+
+int
+ratbag_device_data_asus_get_led_count(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == ASUS);
+	return data->asus.led_count;
+}
+
+int
+ratbag_device_data_asus_get_dpi_count(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == ASUS);
+	return data->asus.dpi_count;
+}
+
+struct dpi_range *
+ratbag_device_data_asus_get_dpi_range(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == ASUS);
+	return data->asus.dpi_range;
+}
+
+int
+ratbag_device_data_asus_is_wireless(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == ASUS);
+	return data->asus.is_wireless;
+}
+
+uint32_t
+ratbag_device_data_asus_get_quirks(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == ASUS);
+	return data->asus.quirks;
 }
