@@ -23,6 +23,8 @@
 
 #pragma once
 
+#include "libratbag-private.h"
+
 struct ratbag_device_data;
 
 struct ratbag_device_data *
@@ -80,12 +82,28 @@ ratbag_device_data_hidpp10_get_led_count(const struct ratbag_device_data *data);
 int
 ratbag_device_data_hidpp20_get_index(const struct ratbag_device_data *data);
 
+int
+ratbag_device_data_hidpp20_get_button_count(const struct ratbag_device_data *data);
+
 int
 ratbag_device_data_hidpp20_get_led_count(const struct ratbag_device_data *data);
 
+int
+ratbag_device_data_hidpp20_get_report_rate(const struct ratbag_device_data *data);
+
 enum hidpp20_quirk
 ratbag_device_data_hidpp20_get_quirk(const struct ratbag_device_data *data);
 
+/* SinoWealth */
+
+/*
+ * NOTE: Don't forget to check for NULL in fields of each device data entry.
+ *
+ * @return List of data for supported devices.
+ */
+const struct list *
+ratbag_device_data_sinowealth_get_supported_devices(const struct ratbag_device_data *data);
+
 /* SteelSeries */
 
 /**
@@ -115,8 +133,58 @@ ratbag_device_data_steelseries_get_dpi_range(const struct ratbag_device_data *da
 int
 ratbag_device_data_steelseries_get_macro_length(const struct ratbag_device_data *data);
 
-int
-ratbag_device_data_steelseries_get_mono_led(const struct ratbag_device_data *data);
+/**
+ * @return Quirk
+ */
+enum steelseries_quirk
+ratbag_device_data_steelseries_get_quirk(const struct ratbag_device_data *data);
 
+/* ASUS */
+
+/**
+ * @return Number of profiles
+ */
 int
-ratbag_device_data_steelseries_get_short_button(const struct ratbag_device_data *data);
+ratbag_device_data_asus_get_profile_count(const struct ratbag_device_data *data);
+
+/**
+ * @return Number of buttons
+ */
+int
+ratbag_device_data_asus_get_button_count(const struct ratbag_device_data *data);
+
+/**
+ * @return Array of button indices, which are used for reading and writing button actions
+ */
+const int8_t *
+ratbag_device_data_asus_get_button_mapping(const struct ratbag_device_data *data);
+
+/**
+ * @return Number of LEDs
+ */
+int
+ratbag_device_data_asus_get_led_count(const struct ratbag_device_data *data);
+
+/**
+ * @return Number of DPI presets
+ */
+int
+ratbag_device_data_asus_get_dpi_count(const struct ratbag_device_data *data);
+
+/**
+ * @return NULL if not set
+ */
+struct dpi_range *
+ratbag_device_data_asus_get_dpi_range(const struct ratbag_device_data *data);
+
+/**
+ * @return 1 if wireless, 0 otherwise
+ */
+int
+ratbag_device_data_asus_is_wireless(const struct ratbag_device_data *data);
+
+/**
+ * @return Quirks bitmask
+ */
+uint32_t
+ratbag_device_data_asus_get_quirks(const struct ratbag_device_data *data);
