@@ -197,13 +197,6 @@ struct ratbag_driver {
 	 */
 	int (*commit)(struct ratbag_device *device);
 
-	/*
-	 * FIXME: This function is deprecated and should be removed. Once
-	 * we've updated all the device drivers to stop using it we'll remove
-	 * it. Look at commit() instead.
-	 */
-	int (*write_profile)(struct ratbag_profile *profile);
-
 	/**
 	 * Called to mark a previously writen profile as active.
 	 *
@@ -212,31 +205,6 @@ struct ratbag_driver {
 	 */
 	int (*set_active_profile)(struct ratbag_device *device, unsigned int index);
 
-	/*
-	 * FIXME: This function is deprecated and should be removed. Once
-	 * we've updated all the device drivers to stop using it we'll remove
-	 * it. Look at commit() instead.
-	 */
-	int (*write_button)(struct ratbag_button *button,
-			    const struct ratbag_button_action *action);
-
-	/*
-	 * FIXME: This function is deprecated and should be removed. Once
-	 * we've updated all the device drivers to stop using it we'll remove
-	 * it. Look at commit() instead.
-	 */
-	int (*write_resolution_dpi)(struct ratbag_resolution *resolution,
-				    int dpi_x, int dpi_y);
-
-	/*
-	 * FIXME: This function is deprecated and should be removed. Once
-	 * we've updated all the device drivers to stop using it we'll remove
-	 * it. Look at commit() instead.
-	 */
-	int (*write_led)(struct ratbag_led *led, enum ratbag_led_mode mode,
-			 struct ratbag_color color, unsigned int ms,
-			 unsigned int brightness);
-
 	/* private */
 	int (*test_probe)(struct ratbag_device *device, const void *data);
 
@@ -462,6 +430,8 @@ ratbag_button_action_match(const struct ratbag_button_action *action,
 		return 0;
 
 	switch (action->type) {
+	case RATBAG_BUTTON_ACTION_TYPE_NONE:
+		return 1;
 	case RATBAG_BUTTON_ACTION_TYPE_BUTTON:
 		return match->action.button == action->action.button;
 	case RATBAG_BUTTON_ACTION_TYPE_KEY:
@@ -469,6 +439,7 @@ ratbag_button_action_match(const struct ratbag_button_action *action,
 	case RATBAG_BUTTON_ACTION_TYPE_SPECIAL:
 		return match->action.special == action->action.special;
 	case RATBAG_BUTTON_ACTION_TYPE_MACRO:
+		/* TODO: check if events match. */
 		return 1;
 	default:
 		break;
@@ -593,8 +564,11 @@ extern struct ratbag_driver hidpp10_driver;
 extern struct ratbag_driver logitech_g300_driver;
 extern struct ratbag_driver logitech_g600_driver;
 extern struct ratbag_driver roccat_driver;
+extern struct ratbag_driver roccat_kone_pure_driver;
+extern struct ratbag_driver roccat_emp_driver;
 extern struct ratbag_driver gskill_driver;
 extern struct ratbag_driver steelseries_driver;
+extern struct ratbag_driver asus_driver;
 extern struct ratbag_driver sinowealth_driver;
 extern struct ratbag_driver sinowealth_nubwo_driver;
 extern struct ratbag_driver openinput_driver;
