@@ -59,11 +59,10 @@ const struct ratbag_test_device sane_device = {
 	.profiles = {
 		{
 		.resolutions = {
-			{ .xres = 100, .yres = 200, .hz = 1000,
-				.dpi_min = 100, .dpi_max = 5000,
-				.report_rates = { 500, 1000 } },
-			{ .xres = 200, .yres = 300, .hz = 1000 },
-			{ .xres = 300, .yres = 400, .hz = 1000 },
+			{ .xres = 100, .yres = 200,
+				.dpi_min = 100, .dpi_max = 5000 },
+			{ .xres = 200, .yres = 300 },
+			{ .xres = 300, .yres = 400 },
 		},
 		.leds = {
 			{ .type = RATBAG_LED_TYPE_SIDE, },
@@ -71,21 +70,24 @@ const struct ratbag_test_device sane_device = {
 		},
 		.active = true,
 		.dflt = false,
+		.report_rates = { 500, 1000 },
+		.hz = 1000,
 		},
 		{
 		.resolutions = {
-			{ .xres = 1100, .yres = 1200, .hz = 2000 },
-			{ .xres = 1200, .yres = 1300, .hz = 2000 },
-			{ .xres = 1300, .yres = 1400, .hz = 2000 },
+			{ .xres = 1100, .yres = 1200 },
+			{ .xres = 1200, .yres = 1300 },
+			{ .xres = 1300, .yres = 1400 },
 		},
 		.active = false,
 		.dflt = true,
+		.hz = 2000,
 		},
 		{
 		.resolutions = {
-			{ .xres = 2100, .yres = 2200, .hz = 3000 },
-			{ .xres = 2200, .yres = 2300, .hz = 3000 },
-			{ .xres = 2300, .yres = 2400, .hz = 3000 },
+			{ .xres = 2100, .yres = 2200 },
+			{ .xres = 2200, .yres = 2300 },
+			{ .xres = 2300, .yres = 2400 },
 		},
 		.leds = {
 			{
@@ -103,6 +105,7 @@ const struct ratbag_test_device sane_device = {
 		},
 		.active = false,
 		.dflt = false,
+		.hz = 3000,
 		},
 	},
 	.destroyed = device_destroyed,
@@ -356,27 +359,30 @@ START_TEST(device_resolutions)
 		.profiles = {
 			{
 			.resolutions = {
-				{ .xres = 100, .yres = 200, .hz = 1000,
-					.dpi_min = 50, .dpi_max = 5000,
-					.report_rates = { 500, 1000 } },
-				{ .xres = 200, .yres = 300, .hz = 1000, .active = true },
-				{ .xres = 300, .yres = 400, .hz = 1000 },
+				{ .xres = 100, .yres = 200,
+					.dpi_min = 50, .dpi_max = 5000 },
+				{ .xres = 200, .yres = 300, .active = true },
+				{ .xres = 300, .yres = 400 },
 			},
 			.active = true,
+			.hz = 1000,
+			.report_rates = { 500, 1000 },
 			},
 			{
 			.resolutions = {
-				{ .xres = 1100, .yres = 1200, .hz = 2000 },
-				{ .xres = 1200, .yres = 1300, .hz = 2000, .active = true },
-				{ .xres = 1300, .yres = 1400, .hz = 2000 },
+				{ .xres = 1100, .yres = 1200 },
+				{ .xres = 1200, .yres = 1300, .active = true },
+				{ .xres = 1300, .yres = 1400 },
 			},
+			.hz = 2000,
 			},
 			{
 			.resolutions = {
-				{ .xres = 2100, .yres = 2200, .hz = 3000 },
-				{ .xres = 2200, .yres = 2300, .hz = 3000, .active = true },
-				{ .xres = 2300, .yres = 2400, .hz = 3000 },
+				{ .xres = 2100, .yres = 2200 },
+				{ .xres = 2200, .yres = 2300, .active = true },
+				{ .xres = 2300, .yres = 2400 },
 			},
+			.hz = 3000,
 			},
 		},
 		.destroyed = device_destroyed,
@@ -392,6 +398,8 @@ START_TEST(device_resolutions)
 		nresolutions = ratbag_profile_get_num_resolutions(p);
 		ck_assert_int_eq(nresolutions, 3);
 
+		rate = ratbag_profile_get_report_rate(p);
+
 		for (j = 0; j < nresolutions; j++) {
 			unsigned int dpis[200];
 			int ndpis = ARRAY_LENGTH(dpis);
@@ -400,7 +408,6 @@ START_TEST(device_resolutions)
 
 			xres = ratbag_resolution_get_dpi_x(res);
 			yres = ratbag_resolution_get_dpi_y(res);
-			rate = ratbag_resolution_get_report_rate(res);
 			is_active = ratbag_resolution_is_active(res);
 
 			ndpis = ratbag_resolution_get_dpi_list(res, dpis, ndpis);
